@@ -998,6 +998,10 @@ class Client:
         check_first_frame: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
         pipeline_mode: str = "BATCH",
+        update_type: str = "SNAPSHOT",
+        is_unlabeled_indexed_dataset: bool = False,
+        seed_dataset_name_for_unlabeled_search: Optional[str] = None,
+        is_anon_mode: bool = False,
     ) -> None:
         """Create or update a dataset with the provided data urls. Dataset must be created using STREAMING pipeline_mode to update.
 
@@ -1014,8 +1018,22 @@ class Client:
             delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
             external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the dataset itself
             pipeline_mode (str, optional): BATCH or STREAMING
+            is_unlabeled_indexed_dataset (bool, optional): Whether this dataset is meant to be used as the seed dataset for unlabeled indexed search.
+            seed_dataset_name_for_unlabeled_search (Optional[str], optional): (Only for unlabeled indexed dataset search). The name of the existing dataset or inf set whose issue elts will be used as the seed for unlabeled indexed search.
+            is_anon_mode (bool, optional): flag to tell aquarium if url images are reachable from public internet or shared bucket. False if reachable, True if Not.
         """
 
+        if is_unlabeled_indexed_dataset:
+            # TODO: is this actually a necessary gate?
+            if pipeline_mode != "BATCH":
+                raise Exception(
+                    f"Unlabeled indexed datasets must be uploaded with BATCH mode. {pipeline_mode} is not supported."
+                )
+            if not seed_dataset_name_for_unlabeled_search:
+                raise Exception(
+                    "seed_dataset_name_for_unlabeled_search must be specified if uploading an unlabeled indexed dataset."
+                )
+
         if pipeline_mode == "STREAMING":
             return self._create_or_update_dataset_streaming(
                 project_id,
@@ -1030,6 +1048,8 @@ class Client:
                 delete_cache_files_after_upload=delete_cache_files_after_upload,
                 check_first_frame=check_first_frame,
                 external_metadata=external_metadata,
+                update_type=update_type,
+                is_anon_mode=is_anon_mode,
             )
 
         assert_valid_name(dataset_id)
@@ -1110,8 +1130,15 @@ class Client:
             "data_url": final_urls,
             "embedding_distance_metric": embedding_distance_metric,
             "embedding_upload_version": 1,
+            "is_anon_mode": is_anon_mode,
         }
 
+        if is_unlabeled_indexed_dataset:
+            payload["is_unlabeled_indexed_dataset"] = True
+            payload[
+                "seed_dataset_name_for_unlabeled_search"
+            ] = seed_dataset_name_for_unlabeled_search
+
         if external_metadata is not None:
             if not isinstance(external_metadata, dict) or (
                 external_metadata and not isinstance(next(iter(external_metadata)), str)
@@ -1138,6 +1165,21 @@ class Client:
                 embeddings_url,
             ]
 
+        if dataset and dataset._temp_frame_asset_file_names:
+            print("Uploading Labeled Frame Assets...")
+            upload_prefix = "{}_assets".format(str(uuid4()))
+            upload_suffix = ".arrow"
+            uploaded_urls = self._upload_rows_from_files(
+                project_id,
+                dataset_id,
+                upload_prefix,
+                upload_suffix,
+                dataset._temp_frame_asset_file_names,
+                delete_after_upload=delete_cache_files_after_upload,
+            )
+            if uploaded_urls:  # not empty list
+                payload["frame_asset_url"] = uploaded_urls
+
         # TODO: Should we just make it not optional, because very few (if any?)
         # people are still using the old data url flow
         if dataset is not None:
@@ -1230,6 +1272,9 @@ class Client:
         check_first_frame: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
         pipeline_mode: str = "BATCH",
+        is_unlabeled_indexed_dataset: bool = False,
+        seed_dataset_name_for_unlabeled_search: Optional[str] = None,
+        is_anon_mode: bool = False,
     ) -> None:
         """Create or update a dataset with the provided data urls. Dataset must be created using STREAMING pipeline_mode to update. (backcompat shortcut for create_or_update_dataset(...))
 
@@ -1246,6 +1291,9 @@ class Client:
             delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
             external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the dataset itself
             pipeline_mode (str, optional): BATCH or STREAMING
+            is_unlabeled_indexed_dataset (bool, optional): Whether this dataset is meant to be used as the seed dataset for unlabeled indexed search.
+            seed_dataset_name_for_unlabeled_search (Optional[str], optional): (Only for unlabeled indexed dataset search). The name of the existing dataset or inf set whose issue elts will be used as the seed for unlabeled indexed search.
+            is_anon_mode (bool, optional): flag to tell aquarium if url images are reachable from public internet or shared bucket. False if reachable, True if Not.
         """
 
         return self.create_or_update_dataset(
@@ -1262,6 +1310,9 @@ class Client:
             check_first_frame=check_first_frame,
             external_metadata=external_metadata,
             pipeline_mode=pipeline_mode,
+            is_unlabeled_indexed_dataset=is_unlabeled_indexed_dataset,
+            seed_dataset_name_for_unlabeled_search=seed_dataset_name_for_unlabeled_search,
+            is_anon_mode=is_anon_mode,
         )
 
     def _create_or_update_dataset_streaming(
@@ -1278,6 +1329,8 @@ class Client:
         delete_cache_files_after_upload: bool = True,
         check_first_frame: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
+        update_type: str = "SNAPSHOT",
+        is_anon_mode: bool = False,
     ) -> None:
         """INTERNAL: Create or update a streaming dataset with the provided data urls. The dataset will be processed by the streaming pipeline and will support updates.
 
@@ -1293,12 +1346,18 @@ class Client:
             preview_first_frame (bool, optional): preview the first frame of the dataset in the webapp before continuing. Requires interaction.
             delete_cache_files_after_upload (bool, optional): flag to turn off automatic deletion of cache files after upload. Useful for ipython notebook users that reload/re-attempt uploads. Defaults to True.
             external_metadata (Optional[Dict[str, Any]], optional): A JSON object that can be used to attach metadata to the dataset itself
+            is_anon_mode (bool, optional): flag to tell aquarium if url images are reachable from public internet or shared bucket. False if reachable, True if Not.
         """
 
         assert_valid_name(dataset_id)
 
         upload_version = str(uuid4())
 
+        if dataset and dataset._temp_frame_asset_file_names:
+            raise Exception(
+                "Mutable streaming datasets do not support mask_data for semseg masks yet."
+            )
+
         if embedding_distance_metric not in ["euclidean", "cosine"]:
             raise Exception("embedding_distance_metric must be euclidean or cosine.")
 
@@ -1401,6 +1460,21 @@ class Client:
             if uploaded_crop_urls:  # not empty list
                 final_urls["crop_embeddings"] = uploaded_crop_urls
 
+        if dataset and dataset._temp_frame_asset_file_names:
+            print("Uploading Labeled Frame Assets...")
+            upload_prefix = "{}_assets".format(str(uuid4()))
+            upload_suffix = ".arrow"
+            uploaded_urls = self._upload_rows_from_files(
+                project_id,
+                dataset_id,
+                upload_prefix,
+                upload_suffix,
+                dataset._temp_frame_asset_file_names,
+                delete_after_upload=delete_cache_files_after_upload,
+            )
+            if uploaded_urls:  # not empty list
+                final_urls["frame_asset_url"] = uploaded_urls
+
         datasets_api_root = self.api_endpoint + "/projects/{}/datasets".format(
             project_id
         )
@@ -1411,6 +1485,8 @@ class Client:
             "embedding_upload_version": 1,
             "upload_version": upload_version,
             "pipeline_mode": "STREAMING",
+            "update_type": update_type,
+            "is_anon_mode": is_anon_mode,
         }
 
         if external_metadata is not None:
@@ -1528,6 +1604,7 @@ class Client:
         delete_cache_files_after_upload: bool = True,
         external_metadata: Optional[Dict[str, Any]] = None,
         pipeline_mode: str = "BATCH",
+        update_type: str = "SNAPSHOT",
     ) -> None:
         """Create or update an inference set with the provided data urls. Dataset + Inferences must be created using STREAMING pipeline_mode to update.
 
@@ -1620,6 +1697,7 @@ class Client:
             "queue_after_dataset": queue_after_dataset,
             "upload_version": upload_version,
             "pipeline_mode": pipeline_mode,
+            "update_type": update_type,
         }
 
         if external_metadata is not None:
@@ -1643,6 +1721,22 @@ class Client:
             )
             if uploaded_urls:  # not empty list
                 payload["embeddings_url"] = uploaded_urls
+
+        if inferences and inferences._temp_frame_asset_file_names:
+            print("Uploading Inference Assets...")
+            upload_prefix = "{}_assets".format(str(uuid4()))
+            upload_suffix = ".arrow"
+            uploaded_urls = self._upload_rows_from_files(
+                project_id,
+                dataset_id,
+                upload_prefix,
+                upload_suffix,
+                inferences._temp_frame_asset_file_names,
+                delete_after_upload=delete_cache_files_after_upload,
+            )
+            if uploaded_urls:  # not empty list
+                payload["frame_asset_url"] = uploaded_urls
+
         elif embeddings_url:
             payload["embeddings_url"] = [
                 embeddings_url,
@@ -1863,3 +1957,38 @@ class Client:
             json=payload,
         )
         raise_resp_exception_error(r)
+
+    def get_frame_ids(self, project_id: str, dataset_id: str) -> List[str]:
+        """Get frame ids in a dataset (for datasets only, not inferences)
+
+        Args:
+            project_id (str): The project id.
+            dataset_id (str): The dataset id.
+        """
+        r = requests_retry.get(
+            f"{self.api_endpoint}/projects/{project_id}/datasets/{dataset_id}/frame_ids/LATEST",
+            headers=self._get_creds_headers(),
+        )
+
+        raise_resp_exception_error(r)
+        result: List[str] = r.json()
+        return result
+
+    def get_frame(
+        self, project_id: str, dataset_id: str, frame_id: str
+    ) -> Dict[str, Any]:
+        """Get frame info (for datasets only, not inferences)
+
+        Args:
+            project_id (str): The project id.
+            dataset_id (str): The dataset id.
+            frame_id (str): The frame id.
+        """
+        r = requests_retry.get(
+            f"{self.api_endpoint}/projects/{project_id}/datasets/{dataset_id}/frame/{frame_id}/LATEST",
+            headers=self._get_creds_headers(),
+        )
+
+        raise_resp_exception_error(r)
+        result: Dict[str, Any] = r.json()
+        return result
