@@ -69,6 +69,38 @@ class LabeledFrame(BaseFrame):
     def _all_label_classes(self) -> List[str]:
         return self._labels._all_label_classes()
 
+    def validate_and_backfill_crop_embeddings(self) -> bool:
+        """Determines if there are frame embeddings but not Crop embeddings
+        Backfills crop embeddings if Frame embeddings exist
+        Returns True if backfill was necessary
+
+        Returns:
+            bool: returns if backfill on frame was necessary
+        """
+
+        # Handle case where Frame embedding is still None
+        if not self.embedding:
+            return False
+
+        if len(self._labels.crop_embeddings) > 0:
+            return False
+
+        return_flag = False
+
+        for label in self._labels.label_data:
+            if label.get("label_type", "") in [
+                "CLASSIFICATION_2D",
+                "CLASSIFICATION_3D",
+            ]:
+                return False
+            label_id = label["uuid"]
+
+            self.add_crop_embedding(
+                label_id=label_id, embedding=[0.0, 0.0, 0.0], model_id="crop_default"
+            )
+            return_flag = True
+        return return_flag
+
     def add_crop_embedding(
         self, *, label_id: str, embedding: List[float], model_id: str = ""
     ) -> None:
@@ -353,17 +385,26 @@ class LabeledFrame(BaseFrame):
         )
 
     def add_label_2d_semseg(
-        self, *, sensor_id: str, label_id: str, mask_url: str
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        mask_url: Optional[str] = None,
+        mask_data: Optional[Any] = None,
     ) -> None:
         """Add a label for 2D semseg.
 
         Args:
             sensor_id (str): sensor_id
             label_id (str): label_id which is unique across datasets and inferences.
-            mask_url (str): URL to the pixel mask png.
+            mask_url (str, optional): URL to the pixel mask png.
+            mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
         """
         self._labels.add_2d_semseg(
-            sensor_id=sensor_id, label_id=label_id, mask_url=mask_url
+            sensor_id=sensor_id,
+            label_id=label_id,
+            mask_url=mask_url,
+            mask_data=mask_data,
         )
 
     # TODO: handle secondary labels
@@ -438,26 +479,33 @@ class LabeledDataset:
     _frame_ids_set: Set[str]
     _label_ids_set: Set[str]
     _label_classes_set: Set[str]
+    _label_class_ids_set: Set[int]
     _frame_summaries: List[LabelFrameSummary]
     _temp_frame_file_names: List[str]
+    _temp_frame_asset_file_names: List[str]
     _temp_frame_embeddings_file_names: List[str]
     _temp_frame_file_names_streaming: List[str]
     _temp_frame_embeddings_file_names_streaming: List[str]
     _temp_crop_file_names_streaming: List[str]
     _temp_crop_embeddings_file_names_streaming: List[str]
     pipeline_mode: str
+    backfilled_crops_flag: bool
 
     def __init__(self, *, pipeline_mode: str = "BATCH") -> None:
         self._frames = []
         self._frame_ids_set = set()
         self._label_ids_set = set()
         self._label_classes_set = set()
+        self._label_class_ids_set = set()
         self._frame_summaries = []
         current_time = datetime.datetime.now()
         self.temp_file_path = create_temp_directory()
         self._temp_frame_prefix = "al_{}_dataset_".format(
             current_time.strftime("%Y%m%d_%H%M%S_%f")
         )
+        self._temp_frame_assets_prefix = "al_{}_dataset_".format(
+            current_time.strftime("%Y%m%d_%H%M%S_%f")
+        )
         self._temp_frame_embeddings_prefix = "al_{}_dataset_frame_embeddings_".format(
             current_time.strftime("%Y%m%d_%H%M%S_%f")
         )
@@ -469,12 +517,14 @@ class LabeledDataset:
         )
         self._temp_frame_file_names = []
         self._temp_frame_embeddings_file_names = []
+        self._temp_frame_asset_file_names = []
 
         self._temp_frame_file_names_streaming = []
         self._temp_frame_embeddings_file_names_streaming = []
         self._temp_crop_file_names_streaming = []
         self._temp_crop_embeddings_file_names_streaming = []
         self.pipeline_mode = pipeline_mode
+        self.backfilled_crops_flag = False
 
     def _cleanup_temp_dir(self) -> None:
         mark_temp_directory_complete(self.temp_file_path)
@@ -553,6 +603,14 @@ class LabeledDataset:
             if embeddings_path:
                 self._temp_frame_embeddings_file_names.append(embeddings_path)
 
+            assets_path = self._save_rows_to_temp(
+                self._temp_frame_assets_prefix,
+                lambda x: self.write_label_assets_to_file(x),
+                mode="wb",
+            )
+            if assets_path:
+                self._temp_frame_asset_file_names.append(assets_path)
+
             self._frames = []
 
     def _flush_to_disk_streaming(self) -> None:
@@ -588,6 +646,14 @@ class LabeledDataset:
         if crop_embeddings_path:
             self._temp_crop_embeddings_file_names_streaming.append(crop_embeddings_path)
 
+        assets_path = self._save_rows_to_temp(
+            self._temp_frame_assets_prefix,
+            lambda x: self.write_label_assets_to_file(x),
+            mode="wb",
+        )
+        if assets_path:
+            self._temp_frame_asset_file_names.append(assets_path)
+
         self._frames = []
 
     def _validate_frame(
@@ -602,23 +668,24 @@ class LabeledDataset:
 
         frame_id = frame_summary["frame_id"]
         primary_task = project_info.get("primary_task")
+
+        # TODO: Should also check this for classification here.
+        # 2D_SEMSEG frames may only have one label of type SEMANTIC_LABEL_URL_2D
         if primary_task == "2D_SEMSEG":
             label_counts = frame_summary["label_counts"]
-
-            # 2D_SEMSEG frames may only have one label of type SEMANTIC_LABEL_URL_2D
-            if (
-                len(label_counts) != 1
-                or label_counts.get("SEMANTIC_LABEL_URL_2D", 0) != 1
-            ):
+            url_count = label_counts.get("SEMANTIC_LABEL_URL_2D", 0)
+            assets_count = label_counts.get("SEMANTIC_LABEL_ASSET_2D", 0)
+            if url_count + assets_count != 1:
                 extra_labels = filter(
-                    lambda x: x != "SEMANTIC_LABEL_URL_2D", label_counts.keys()
+                    lambda x: x != "SEMANTIC_LABEL_URL_2D"
+                    and x != "SEMANTIC_LABEL_ASSET_2D",
+                    label_counts.keys(),
                 )
                 issue_text = (
                     "has no labels"
                     if not len(label_counts)
                     else f"has label types of {list(extra_labels)}"
                 )
-
                 raise Exception(
                     f"Frame {frame_id} {issue_text}. Dataset frames for 2D_SEMSEG projects must have exactly one 2d_semseg label"
                 )
@@ -631,6 +698,10 @@ class LabeledDataset:
         """
         for frame_summary in self._frame_summaries:
             self._validate_frame(frame_summary, project_info)
+        if self.backfilled_crops_flag:
+            print(
+                "WARNING: One or more Frames contained Frame Embeddings with Crops with no Crop Embeddings. All crops without Crop Embeddings have been backfilled with a default Embedding Vector"
+            )
 
     def add_frame(self, frame: LabeledFrame) -> None:
         """Add a LabeledFrame to this dataset.
@@ -644,15 +715,17 @@ class LabeledDataset:
         if frame.frame_id in self._frame_ids_set:
             raise Exception("Attempted to add duplicate frame id.")
 
-        duplicate_label_ids = frame._label_ids_set & self._label_ids_set
+        duplicate_label_ids = frame._labels._label_ids_set & self._label_ids_set
         if duplicate_label_ids:
             raise Exception(
                 f"Attempted to add duplicate label id(s): {duplicate_label_ids}"
             )
 
+        self.backfilled_crops_flag = frame.validate_and_backfill_crop_embeddings()
         self._frames.append(frame)
         self._frame_ids_set.add(frame.frame_id)
-        self._label_ids_set.update(frame._label_ids_set)
+        self._label_ids_set.update(frame._labels._label_ids_set)
+        self._label_class_ids_set.update(frame._labels._seen_label_class_ids)
         self._label_classes_set.update(frame._all_label_classes())
         self._frame_summaries.append(frame._to_summary())
         if len(self._frames) >= MAX_FRAMES_PER_BATCH:
@@ -669,6 +742,36 @@ class LabeledDataset:
             row["label_data"] = frame._labels.to_dict()["label_data"]
             filelike.write(json.dumps(row) + "\n")
 
+    # TODO: Does this need a streaming equivalent?
+    def write_label_assets_to_file(self, filelike: IO[Any]) -> None:
+        """Write the frame's label assets to a text filelike object (File handle, StringIO, etc.)
+
+        Args:
+            filelike (filelike): The destination file-like to write to.
+        """
+        count = len(self._frames)
+        with_label_assets_count = len(
+            [frame for frame in self._frames if len(frame._labels.label_assets) > 0]
+        )
+        if with_label_assets_count == 0:
+            return
+
+        frame_ids = np.empty((count), dtype=object)
+        label_assets = np.empty((count), dtype=object)
+
+        for i, frame in enumerate(self._frames):
+            frame_ids[i] = frame.frame_id
+            label_assets[i] = frame._labels.label_assets
+
+        df = pd.DataFrame(
+            {"frame_ids": pd.Series(frame_ids), "label_assets": pd.Series(label_assets)}
+        )
+
+        arrow_data = pa.Table.from_pandas(df)
+        writer = pa.ipc.new_file(filelike, arrow_data.schema, use_legacy_format=False)
+        writer.write(arrow_data)
+        writer.close()
+
     def write_embeddings_to_file(self, filelike: IO[Any]) -> None:
         """Write the frame's embeddings to a text filelike object (File handle, StringIO, etc.)
 
