@@ -572,7 +572,7 @@ class BaseFrame:
         if not isinstance(overlay_id, str):
             raise Exception("overlay ids must be strings")
 
-        sensor_metadata: dict[str, Union[str, int]] = {"overlay_id": overlay_id}
+        sensor_metadata: Dict[str, Union[str, int]] = {"overlay_id": overlay_id}
         if width is not None:
             if not isinstance(width, int):
                 raise Exception("width must be an int")
