@@ -338,16 +338,27 @@ class InferencesFrame(InferenceLabelSet):
         )
 
     def add_inference_2d_semseg(
-        self, *, sensor_id: str, label_id: str, mask_url: str
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        mask_url: Optional[str] = None,
+        mask_data: Optional[Any] = None,
     ) -> None:
         """Add an inference for 2D semseg.
 
         Args:
             sensor_id (str): sensor_id
             label_id (str): label_id which is unique across datasets and inferences.
-            mask_url (str): URL to the pixel mask png.
+            mask_url (str, optional): URL to the pixel mask png.
+            mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
         """
-        self.add_2d_semseg(sensor_id=sensor_id, label_id=label_id, mask_url=mask_url)
+        self.add_2d_semseg(
+            sensor_id=sensor_id,
+            label_id=label_id,
+            mask_url=mask_url,
+            mask_data=mask_data,
+        )
 
     def add_inference_2d_classification(
         self,
@@ -413,8 +424,10 @@ class Inferences:
     _frame_ids_set: Set[str]
     _label_ids_set: Set[str]
     _label_classes_set: Set[str]
+    _label_class_ids_set: Set[int]
     _frame_summaries: List[InferenceFrameSummary]
     _temp_frame_file_names: List[str]
+    _temp_frame_asset_file_names: List[str]
     _temp_frame_embeddings_file_names: List[str]
 
     def __init__(self) -> None:
@@ -426,17 +439,22 @@ class Inferences:
         self._frame_ids_set = set()
         self._label_ids_set = set()
         self._label_classes_set = set()
+        self._label_class_ids_set = set()
         self._frame_summaries = []
         current_time = datetime.datetime.now()
         self.temp_file_path = create_temp_directory()
         self._temp_frame_prefix = "al_{}_inference_".format(
             current_time.strftime("%Y%m%d_%H%M%S_%f")
         )
+        self._temp_frame_assets_prefix = "al_{}_dataset_".format(
+            current_time.strftime("%Y%m%d_%H%M%S_%f")
+        )
         self._temp_frame_embeddings_prefix = "al_{}_inference_embeddings_".format(
             current_time.strftime("%Y%m%d_%H%M%S_%f")
         )
         self._temp_frame_file_names = []
         self._temp_frame_embeddings_file_names = []
+        self._temp_frame_asset_file_names = []
 
     def _cleanup_temp_dir(self) -> None:
         mark_temp_directory_complete(self.temp_file_path)
@@ -492,6 +510,15 @@ class Inferences:
         )
         if embeddings_path:
             self._temp_frame_embeddings_file_names.append(embeddings_path)
+
+        assets_path = self._save_rows_to_temp(
+            self._temp_frame_assets_prefix,
+            lambda x: self.write_label_assets_to_file(x),
+            mode="wb",
+        )
+        if assets_path:
+            self._temp_frame_asset_file_names.append(assets_path)
+
         self._frames = []
 
     def _validate_frame(
@@ -506,25 +533,26 @@ class Inferences:
 
         frame_id = frame_summary["frame_id"]
         primary_task = project_info.get("primary_task")
+
+        # TODO: Should also check this for classification here.
+        # 2D_SEMSEG frames may only have one label of type SEMANTIC_LABEL_URL_2D
         if primary_task == "2D_SEMSEG":
             label_counts = frame_summary["label_counts"]
-
-            # 2D_SEMSEG frames may only have one label of type SEMANTIC_LABEL_URL_2D
-            if (
-                len(label_counts) != 1
-                or label_counts.get("SEMANTIC_LABEL_URL_2D", 0) != 1
-            ):
+            url_count = label_counts.get("SEMANTIC_LABEL_URL_2D", 0)
+            assets_count = label_counts.get("SEMANTIC_LABEL_ASSET_2D", 0)
+            if url_count + assets_count != 1:
                 extra_labels = filter(
-                    lambda x: x != "SEMANTIC_LABEL_URL_2D", label_counts.keys()
+                    lambda x: x != "SEMANTIC_LABEL_URL_2D"
+                    and x != "SEMANTIC_LABEL_ASSET_2D",
+                    label_counts.keys(),
                 )
                 issue_text = (
                     "has no labels"
                     if not len(label_counts)
-                    else f"has inference types of {list(extra_labels)}"
+                    else f"has label types of {list(extra_labels)}"
                 )
-
                 raise Exception(
-                    f"Frame {frame_id} {issue_text}. Inferences frames for 2D_SEMSEG projects must have exactly one 2d_semseg inference"
+                    f"Frame {frame_id} {issue_text}. Inferences frames for 2D_SEMSEG projects must have exactly one 2d_semseg inference "
                 )
 
         project_custom_metrics_names = [
@@ -587,6 +615,7 @@ class Inferences:
         self._frames.append(frame)
         self._frame_ids_set.add(frame.frame_id)
         self._label_ids_set.update(frame._label_ids_set)
+        self._label_class_ids_set.update(frame._seen_label_class_ids)
         self._label_classes_set.update(frame._all_label_classes())
         self._frame_summaries.append(frame._to_summary())
         if len(self._frames) >= MAX_FRAMES_PER_BATCH:
@@ -603,6 +632,35 @@ class Inferences:
             row = frame.to_dict()
             filelike.write(json.dumps(row) + "\n")
 
+    def write_label_assets_to_file(self, filelike: IO[Any]) -> None:
+        """Write the frame's label assets to a text filelike object (File handle, StringIO, etc.)
+
+        Args:
+            filelike (filelike): The destination file-like to write to.
+        """
+        count = len(self._frames)
+        with_label_assets_count = len(
+            [frame for frame in self._frames if len(frame.label_assets) > 0]
+        )
+        if with_label_assets_count == 0:
+            return
+
+        frame_ids = np.empty((count), dtype=object)
+        label_assets = np.empty((count), dtype=object)
+
+        for i, frame in enumerate(self._frames):
+            frame_ids[i] = frame.frame_id
+            label_assets[i] = frame.label_assets
+
+        df = pd.DataFrame(
+            {"frame_ids": pd.Series(frame_ids), "label_assets": pd.Series(label_assets)}
+        )
+
+        arrow_data = pa.Table.from_pandas(df)
+        writer = pa.ipc.new_file(filelike, arrow_data.schema, use_legacy_format=False)
+        writer.write(arrow_data)
+        writer.close()
+
     def write_embeddings_to_file(self, filelike: IO[Any]) -> None:
         """Write the frame's embeddings to a text filelike object (File handle, StringIO, etc.)
 
