@@ -535,3 +535,30 @@ class IssueManager:
         )
 
         raise_resp_exception_error(r)
+
+    def update_elements_status(
+        self, issue_id: str, element_ids: List[str], new_status: str
+    ) -> None:
+        """Update issue elements status.
+
+        Args:
+            issue_id (str): The issue id.
+            new_status (str): The new status. ("unstarted", "done")
+        """
+
+        payload = {
+            "edit_type": "update",
+            "element_type": None,
+            "elements": [{"id": element_id} for element_id in element_ids],
+            "new_status": new_status,
+        }
+
+        url = "/projects/{}/issues/{}/elements".format(self.project_id, issue_id)
+
+        r = requests.patch(
+            self.client.api_endpoint + url,
+            headers=self.client._get_creds_headers(),
+            json=payload,
+        )
+
+        raise_resp_exception_error(r)
