@@ -5,13 +5,22 @@ The label set classes.
 
 import datetime
 import json
-from io import IOBase
+from io import IOBase, BytesIO
 from tempfile import NamedTemporaryFile
 import pyarrow as pa
 import numpy as np
 import pandas as pd
 from typing import Any, Optional, Union, Callable, List, Dict, Tuple, Set, IO, cast
 from typing_extensions import Literal
+from uuid import uuid4
+
+HAS_PILLOW = False
+try:
+    from PIL import Image  # type: ignore
+
+    HAS_PILLOW = True
+except ImportError:
+    HAS_PILLOW = False
 
 
 from .util import (
@@ -29,6 +38,7 @@ from .util import (
     InferenceFrameSummary,
     LabelFrameSummary,
     BaseLabelEntryDict,
+    BaseLabelAssetDict,
 )
 
 CustomMetricsEntry = Union[float, List[List[Union[float, int]]]]
@@ -46,8 +56,10 @@ class BaseLabelSet:
     frame_id: str
     crop_type: CropType
     label_data: List[BaseLabelEntryDict]
+    label_assets: List[BaseLabelAssetDict]
     _label_ids_set: Set[str]
     crop_embeddings: List[CropEmbeddingDict]
+    _seen_label_class_ids: Set[int]
 
     def __init__(self, *, frame_id: str, crop_type: CropType) -> None:
         if not isinstance(frame_id, str):
@@ -59,8 +71,10 @@ class BaseLabelSet:
         self.frame_id = frame_id
         self.crop_type = crop_type
         self.label_data = []
+        self.label_assets = []
         self.crop_embeddings = []
         self._label_ids_set = set()
+        self._seen_label_class_ids = set()
 
     def add_crop_embedding(
         self, *, label_id: str, embedding: List[float], model_id: str = ""
@@ -563,13 +577,23 @@ class BaseLabelSet:
         )
         self._label_ids_set.add(label_id)
 
-    def add_2d_semseg(self, *, sensor_id: str, label_id: str, mask_url: str) -> None:
-        """Add an inference for 2D semseg.
+    # TODO: Do we want to expect column major or row major?
+    def add_2d_semseg(
+        self,
+        *,
+        sensor_id: str,
+        label_id: str,
+        mask_url: Optional[str] = None,
+        mask_data: Optional[Any] = None,
+    ) -> None:
+        """
+        Add an inference for 2D semseg. These should provide either a mask_url or a mask_data.
 
         Args:
             sensor_id (str): sensor_id
             label_id (str): label_id which is unique across datasets and inferences.
-            mask_url (str): URL to the pixel mask png.
+            mask_url (str, optional): URL to the pixel mask png.
+            mask_data (ndarray, optional): ndarray of pixel mask data, shape [height, width]
         """
 
         if not isinstance(label_id, str):
@@ -577,17 +601,70 @@ class BaseLabelSet:
         if "/" in label_id:
             raise Exception("label ids cannot contain slashes (/)")
 
-        self.label_data.append(
-            {
-                "uuid": label_id,
-                "linked_labels": [],
-                "label": "__mask",
-                "label_coordinate_frame": sensor_id,
-                "label_type": "SEMANTIC_LABEL_URL_2D",
-                "attributes": {"url": mask_url},
-            }
-        )
-        self._label_ids_set.add(label_id)
+        if mask_data is None and mask_url is None:
+            raise Exception("Please provide either a mask_data or a mask_url")
+
+        if mask_url is not None:
+            self.label_data.append(
+                {
+                    "uuid": label_id,
+                    "linked_labels": [],
+                    "label": "__mask",
+                    "label_coordinate_frame": sensor_id,
+                    "label_type": "SEMANTIC_LABEL_URL_2D",
+                    "attributes": {"url": mask_url},
+                }
+            )
+            self._label_ids_set.add(label_id)
+
+        elif mask_data is not None:
+            if not HAS_PILLOW:
+                raise Exception(
+                    "To use mask_data, please install with additional image dependencies using `pip install aquariumlearning[img]`"
+                )
+
+            if not isinstance(mask_data, (np.ndarray, np.generic)):
+                raise Exception("mask_data must be a numpy ndarray")
+
+            if not np.issubdtype(mask_data.dtype, np.integer):
+                raise Exception("mask_data must have an integer dtype")
+
+            if len(mask_data.shape) != 2:
+                raise Exception(
+                    "mask_data must be a 2d ndarray of shape [height, width]"
+                )
+
+            # TODO: Validate
+            # TODO: Is png encoding too slow? Would something like pycocotools RLE be better?
+            unique_label_ids = np.unique(mask_data)
+            for v in unique_label_ids:
+                self._seen_label_class_ids.add(cast(int, v))
+
+            # Compress to a png for later usage, and immediate compression
+            png_img = Image.fromarray(mask_data.astype("uint8"))
+            png_io = BytesIO()
+            png_img.save(
+                png_io, format="PNG", compress_level=6
+            )  # 0-9, defualt is 6, we favor slightly faster
+            png_io.seek(0)
+            png_bytes = png_io.read()
+
+            asset_uuid = str(uuid4())
+            self.label_assets.append(
+                {"uuid": asset_uuid, "asset_type": "SEMSEG_MASK", "value": png_bytes}
+            )
+
+            self.label_data.append(
+                {
+                    "uuid": label_id,
+                    "linked_labels": [],
+                    "label": "__mask",
+                    "label_coordinate_frame": sensor_id,
+                    "label_type": "SEMANTIC_LABEL_ASSET_2D",
+                    "attributes": {"asset_uuid": asset_uuid},
+                }
+            )
+            self._label_ids_set.add(label_id)
 
     def add_2d_classification(
         self,
