@@ -320,6 +320,7 @@ LabelType = Literal[
     "KEYPOINTS_2D",
     "POLYGON_LIST_2D",
     "SEMANTIC_LABEL_URL_2D",
+    "SEMANTIC_LABEL_ASSET_2D",
     "CLASSIFICATION_2D",
     "CLASSIFICATION_3D",
 ]
@@ -344,6 +345,15 @@ class BaseLabelEntryDict(TypedDict):
     attributes: LabelAttrs
 
 
+LabelAssetType = Literal["SEMSEG_MASK"]
+
+
+class BaseLabelAssetDict(TypedDict):
+    uuid: str
+    asset_type: LabelAssetType
+    value: Any
+
+
 class GtLabelEntryDict(BaseLabelEntryDict):
     pass
 
